VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MultiModem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


Private WithEvents SerialPort As MSComm
Attribute SerialPort.VB_VarHelpID = -1


Private current As String

Private responseQ As Queue
Private incomingMsgQ As Queue
Private listMsgQ As Queue
Private listMsgFlag As Boolean


'************************************************
'* Private Sub Class_Initialize()
'* Summary:     Occurs when class is initialized
'*              Creates a new MSComm object, initializes it
'************************************************
Private Sub Class_Initialize()

    'Create new MSComm object
    Set SerialPort = New MSComm
        
    '115200 baud rate, no parity, 8 data bits, 1 stop bit
    SerialPort.Settings = "115200,N,8,1"
    
    'No handshaking
    SerialPort.Handshaking = comNone
    
    'Tell the control to read entire buffer when Input is used
    SerialPort.InputLen = 0
    
    'Enables onComm event to trigger when RThreshold bytes are in the input buffer
    SerialPort.RThreshold = 1
    
    'Text input mode
    SerialPort.InputMode = comInputModeText
    
    'Set current string to empty
    current = ""
    
    'Set up queues
    Set responseQ = New Queue
    responseQ.Initialize 50
    
    Set incomingMsgQ = New Queue
    incomingMsgQ.Initialize 50
    
    Set listMsgQ = New Queue
    listMsgQ.Initialize 50
    
    listMsgFlag = False

End Sub



'************************************************
'* Private Sub Class_Terminate()
'* Summary:     Occurs when class terminates
'*              Disposes MSComm object
'************************************************
Private Sub Class_Terminate()

    'Dispose of MSComm object
    Set SerialPort = Nothing

End Sub



'************************************************
'* Public Function Connect(port As Integer) As Boolean
'* Summary:     Attempts to connect to the serial port specified by port
'* Parameters:  port - port number to connect to
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function Connect(port As Integer) As Boolean

    On Error Resume Next
    
    'Assign port number to CommPort property
    SerialPort.CommPort = port
    
    'Try opening port
    SerialPort.PortOpen = True
    
    'Return True if port is open, False if port is not open
    Connect = SerialPort.PortOpen

End Function



'************************************************
'* Public Function Disconnect() As Boolean
'* Summary:     Attempts to disconnect from the currently open serial port
'* Returns:     True if successful or serial port already closed,
'*              False if port is still open
'************************************************
Public Function Disconnect() As Boolean

    On Error Resume Next
    
    'Try closing the port
    SerialPort.PortOpen = False
    
    'Return True if port is closed, False if port is open
    Disconnect = Not SerialPort.PortOpen

End Function



'************************************************
'* Public Function GetClock(ByRef clock As String) As Boolean
'* Summary:     Retrieves clock reading from modem
'* Parameters:  clock - clock reading if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetClock(ByRef clock As String) As Boolean

    Dim response As String
    
    GetClock = False
    
    If Not SendATCommand("AT+CCLK?", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        clock = Trim(Left(response, Len(response) - 2))
        If Len(clock) > 9 Then clock = Mid(clock, 9, Len(clock) - 9)
        GetClock = True
    End If

End Function



'************************************************
'* Public Function SetClock(clock As String) As Boolean
'* Summary:     Sets clock on modem
'* Parameters:  clock - time to set modem to
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function SetClock(clock As String) As Boolean

    Dim response As String
    
    SetClock = False
    
    If Not SendATCommand("AT+CCLK=""" & clock & """", response, 0) Then Exit Function

    SetClock = True

End Function



'************************************************
'* Public Function GetManufacturerID(ByRef mid As String) As Boolean
'* Summary:     Retrieves Manufacturer ID of modem
'* Parameters:  manufacturer - Manufacturer ID if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetManufacturerID(ByRef manufacturer As String) As Boolean

    Dim response As String
    
    GetManufacturerID = False
    
    If Not SendATCommand("AT+CGMI", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        manufacturer = Trim(Left(response, Len(response) - 2))
        GetManufacturerID = True
    End If

End Function



'************************************************
'* Public Function GetModelID(ByRef model As String) As Boolean
'* Summary:     Retrieves Model ID of modem
'* Parameters:  model - Model ID if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetModelID(ByRef model As String) As Boolean

    Dim response As String
    
    GetModelID = False
    
    If Not SendATCommand("AT+CGMM", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        model = Trim(Left(response, Len(response) - 2))
        GetModelID = True
    End If

End Function



'************************************************
'* Public Function GetRevisionID(ByRef revision As String) As Boolean
'* Summary:     Retrieves Revision ID of modem
'* Parameters:  revision - Revision ID if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetRevisionID(ByRef revision As String) As Boolean

    Dim response As String
    
    GetRevisionID = False
    
    If Not SendATCommand("AT+CGMR", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        revision = Trim(Left(response, Len(response) - 2))
        GetRevisionID = True
    End If

End Function



'************************************************
'* Public Function GetSerialNumber(ByRef serial As String) As Boolean
'* Summary:     Retrieves Serial Number of modem
'* Parameters:  serial - Serial Number if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetSerialNumber(ByRef serial As String) As Boolean

    Dim response As String
    
    GetSerialNumber = False
    
    If Not SendATCommand("AT+CGSN", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        serial = Trim(Left(response, Len(response) - 2))
        GetSerialNumber = True
    End If

End Function



'************************************************
'* Public Function GetSubscriberNumber(ByRef subscriber As String) As Boolean
'* Summary:     Retrieves Subscriber Number of modem
'* Parameters:  subscriber - Subscriber Number if successful
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function GetSubscriberNumber(ByRef subscriber As String) As Boolean

    Dim response As String
    
    GetSubscriberNumber = False
    
    If Not SendATCommand("AT+CNUM", response, 1) Then Exit Function
    
    If InStr(response, vbCrLf) > 0 Then
        subscriber = Trim(Left(response, Len(response) - 2))
        subscriber = Mid(subscriber, InStr(subscriber, """,") + 2)
        subscriber = Left(subscriber, InStr(subscriber, ",") - 1)
        subscriber = Mid(subscriber, 2, Len(subscriber) - 2)
        GetSubscriberNumber = True
    End If

End Function



'************************************************
'* Public Function SetCharacterSet(characterset As String) As Boolean
'* Summary:     Retrieves Subscriber Number of modem
'* Parameters:  characterset - Character Set to use
'*              AT+CSCS=? will give a list of valid character sets, these include:
'*              "GSM","PCCP437","CUSTOM","HEX"
'*              "PCCP437" is the default
'* Returns:     True if successful, False if unsuccessful
'************************************************
Public Function SetCharacterSet(characterset As String) As Boolean

    Dim response As String
    
    SetCharacterSet = False
    
    If Not SendATCommand("AT+CSCS=""" & characterset & """", response, 0) Then Exit Function

    SetCharacterSet = True

End Function



Public Function DeleteMessage(index As Integer) As Boolean

    Dim response As String
    
    DeleteMessage = False
    
    If Not SendATCommand("AT+CMGD=" & index, response, 0) Then Exit Function

    DeleteMessage = True

End Function



Public Function ReadMessage(index As Integer, ByRef unread As Boolean, ByRef number As String, ByRef timestamp As String, ByRef msg As String)

    Dim line1 As String
    Dim pos As Integer
    Dim s As String
    
    ReadMessage = False
    
    If Not SendATCommand("AT+CMGR=" & index, line1, 2) Then Exit Function
    
    'If InStr(line1, "+CMGL:") <> 1 Then Exit Function
    
    'Read whether message has been read or not
    line1 = Mid(line1, 8)
    pos = InStr(line1, ",")
    s = Left(line1, pos - 1)
    If s = """REC READ""" Then
        unread = False
    ElseIf s = """REC UNREAD""" Then
        unread = True
    Else
        'Shouldn't be here
        unread = True
    End If
        
    'Read phone number
    line1 = Mid(line1, pos + 1)
    pos = InStr(line1, ",")
    s = Left(line1, pos - 1)
    number = Mid(s, 2, Len(s) - 2)      'Strip quotes
    
    'This parameter appears to be blank and not used
    line1 = Mid(line1, pos + 1)
    pos = InStr(line1, ",")
    s = Left(line1, pos - 1)
    
    'Read time stamp (last parameter)
    line1 = Mid(line1, pos + 1)
    pos = InStr(line1, vbCrLf)
    s = Left(line1, pos - 1)
    timestamp = Mid(s, 2, Len(s) - 2)   'Strip quotes
    
    'Read message contents
    msg = Mid(line1, pos + 2)
    If InStr(msg, vbCrLf) = 0 Then Exit Function
    msg = Left(msg, Len(msg) - 2)
    
    ReadMessage = True

End Function



Public Function CheckMessages(ByRef index As Integer) As Boolean
        
    Dim cmti As String
    Dim pos As Integer
    
    CheckMessages = False
        
    If incomingMsgQ.Count >= 1 Then
    
        cmti = incomingMsgQ.Remove
        pos = InStr(cmti, """,")
        cmti = Mid(cmti, pos + 2)
        cmti = Left(cmti, Len(cmti) - 2)
        index = CInt(cmti)
        
        CheckMessages = True
    
    End If

End Function


Public Function ListMessages()

    SerialPort.Output = "AT+CMGL=""ALL""" & vbCr

End Function


Public Function CheckListMessages(ByRef index As Integer, ByRef unread As Boolean, ByRef number As String, ByRef timestamp As String, ByRef msg As String)

    Dim line1 As String, line2 As String, s As String
    Dim pos As Integer
    
    CheckListMessages = False
    
    If listMsgQ.Count >= 2 Then
    
        line1 = listMsgQ.Remove
        line2 = listMsgQ.Remove
        
        If InStr(line1, "+CMGL:") <> 1 Then Exit Function
        
        'Read index of message
        line1 = Mid(line1, 8)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        index = CInt(s)
        
        'Read whether message has been read or not
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        If s = """REC READ""" Then
            unread = False
        ElseIf s = """REC UNREAD""" Then
            unread = True
        Else
            'Shouldn't be here
            unread = True
        End If
            
        'Read phone number
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        number = Mid(s, 2, Len(s) - 2)      'Strip quotes
        
        'This parameter appears to be blank and not used
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        
        'Read time stamp (last parameter)
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, vbCrLf)
        s = Left(line1, pos - 1)
        timestamp = Mid(s, 2, Len(s) - 2)   'Strip quotes
        
        'Read message contents
        msg = line2
        If InStr(msg, vbCrLf) = 0 Then Exit Function
        msg = Left(msg, Len(msg) - 2)
        
        CheckListMessages = True
    
    End If

End Function




Public Function SendMessage(phonenumber As String, msg As String) As Boolean

    Dim prompt As String, response As String, ack As String

    SendMessage = False
    
    If Not SerialPort.PortOpen Then Exit Function

    If Not SendATCommand("AT+CMGF=1", response, 0) Then Exit Function

    responseQ.Clear
    
    SerialPort.Output = "AT+CMGS=" & phonenumber & vbCr
    
    While responseQ.Count < 1
        DoEvents
    Wend
    
    prompt = responseQ.Remove
    
    If prompt = "> " Then
    
        SerialPort.Output = msg & Chr(26)
        
        While responseQ.Count < 2
            DoEvents
        Wend
        
        response = responseQ.Remove
        ack = responseQ.Remove
        
        If InStr(response, "+CMGS:") > 0 And ack = "OK" & vbCrLf Then SendMessage = True
    
    End If

End Function



Public Function SendMessageTPDU(phonenumber As String, msg As String) As Boolean

    Dim prompt As String, response As String, ack As String
    
    Dim fmsg As String
    Dim i As Integer
    

    SendMessageTPDU = False
    
    If Not SerialPort.PortOpen Then Exit Function
    
    If Not SendATCommand("AT+CMGF=0", response, 0) Then Exit Function
    
    
    
    
    fmsg = "0001030A91"
    For i = 0 To 4
        fmsg = fmsg & Mid(phonenumber, i * 2 + 2, 1) & Mid(phonenumber, i * 2 + 1, 1)
    Next i
    
    fmsg = fmsg & "00F5"
    
    If (Len(msg) \ 2) < 16 Then
        fmsg = fmsg & "0" & Hex(Len(msg) \ 2)
    Else
        fmsg = fmsg & Hex(Len(msg) \ 2)
    End If
    
    fmsg = fmsg & msg
    
    
    
    
    responseQ.Clear
    
    SerialPort.Output = "AT+CMGS=" & ((Len(fmsg) - 2) \ 2) & vbCr
    
    
    While responseQ.Count < 1
        DoEvents
    Wend
    
    prompt = responseQ.Remove
    
    If prompt = "> " Then
    
        SerialPort.Output = fmsg & Chr(26)
        
        While responseQ.Count < 2
            DoEvents
        Wend
        
        response = responseQ.Remove
        ack = responseQ.Remove
        
        If InStr(response, "+CMGS:") > 0 And ack = "OK" & vbCrLf Then SendMessageTPDU = True
    
    End If

    SendATCommand "AT+CMGF=1", response, 0

End Function



Public Function SendATCommand(command As String, ByRef response As String, lines As Integer) As Boolean

    Dim ack As String
    Dim i As Integer
    
    SendATCommand = False
    
    If Not SerialPort.PortOpen Then Exit Function
    
    responseQ.Clear
    
    SerialPort.Output = command & vbCr
    
    While responseQ.Count < lines + 1
        DoEvents
    Wend
    
    response = ""
    For i = 1 To lines
        response = response & responseQ.Remove
    Next i
    
    ack = responseQ.Remove
    
    If ack = "OK" & vbCrLf Then SendATCommand = True

End Function


Private Sub SerialPort_OnComm()

    Dim s As String
    Dim loc As Integer
    
    s = SerialPort.Input

    'Process all current input
    While Len(s) > 0
    
        current = current & s
        s = ""
         
        If InStr(current, "> ") = 1 Then
        
            responseQ.Add "> "
            
            current = Mid(current, 3)
        
        End If
        
        If InStr(current, vbCrLf) > 0 Then
        
            loc = InStr(current, vbCrLf)
            s = Mid(current, loc + 2)
            current = Left(current, loc + 1)

            If InStr(current, "RING") = 1 Then
                'Ignore for now, could queue or do something else
            ElseIf InStr(current, "NO CARRIER") = 1 Then
                'Ignore for now
            ElseIf InStr(current, "+CMTI:") = 1 Then
                incomingMsgQ.Add current
            ElseIf InStr(current, "+CMGL:") = 1 Then
                listMsgQ.Add current
                listMsgFlag = True
            ElseIf listMsgFlag Then
                listMsgQ.Add current
                listMsgFlag = False
            Else
                If current <> vbCrLf Then
                    responseQ.Add current
                End If
            End If
            
            current = ""
        
        End If
        
    Wend

End Sub






